
#include <windows.h>
#include <openssl\ssl.h>
#include <openssl\err.h>
#include <libsnet\snet.h>
#include "log.h"
#include "CosignMsg.h"
#include "CosignStructs.h"
#include "network.h"

int
teardown_conn( struct connlist *cur );

int
PopulateConnList( COSIGNSETTINGS* cs, struct connlist **cl, struct hostent* he )
{
    struct connlist		*clnew, **cur;
	int					i;

    /* preserve address order as returned from DNS */
    /* actually, here we will randomize for "load balancing" */ 
    cur = cl;
	
    for ( i = 0; he->h_addr_list[ i ] != NULL; i++ ) {
		if ( (clnew = (struct connlist *)malloc( sizeof( struct connlist ) ) ) == NULL ) {
			CosignLog( CS_ERROR, "PopulateConnList: malloc() failed, could not create a connection list.\n" );
			return( -1 );
		}
		
		memset( &clnew->conn_sin, 0, sizeof( struct sockaddr_in ));
		clnew->conn_sin.sin_family = AF_INET;
		clnew->conn_sin.sin_port = htons( cs->cs_port );
		memcpy( &clnew->conn_sin.sin_addr.s_addr, he->h_addr_list[ i ], (unsigned int)he->h_length );
		clnew->conn_sn = NULL;
		*cur = clnew;
		cur = &clnew->conn_next;
    }

    *cur = NULL;
	return( 0 );
}

/*
 * CreateConnPool()
 *
 * struct connpool **cp - The holder of all things connectable.
 * char* host - DNS name of the central weblogin server.
 * int numConnections - Number of connections in the connection pool that should
 * be made.
 *
 * returns int - Returns the number of connections made.  If this item is 0, no connections
 * could be made.  If this number is -1, there was a system or socket error.
 */

int
CreateConnPool( COSIGNSETTINGS*	cs, struct connpool **cp, char* host, int numConnections )
{

	int	i;
	int	numConnectionsMade = 0;
	struct connpool	**cur;
	struct hostent	*he;

	cur = cp;

	if ( numConnections < 0 ) {
		return( 0 );
	}
	CSDEBUG1( CS_INFO, "The host is: %s\n", host );
	if ( (he = gethostbyname( host )) == NULL ) {
		CosignLog( (DWORD)WSAGetLastError(), "CreateConnPool::getaddrinfo( %s ) failed.\n", host );
		return( -1 );
	}

	CSDEBUG0( CS_INFO, "Creating connection pool...\n" );
	for( i = 0; i < numConnections; i++ ) {
		*cur = (struct connpool*)malloc( sizeof( struct connpool) );
		(*cur)->cp_locked = 0;
		if ( PopulateConnList( cs, &(*cur)->cp_cl, he ) == 0 ) {
			numConnectionsMade++;
			cur = &(*cur)->cp_next;
		} else {
			/* Could not make a connection list for this item.
			 * Freeing the memory so we don't leave an "empty slot" in connection
			 * pool. */
            CosignLog( CS_ERROR, "CreateConnPool: Failure while making connection list number %d.\n", i );
			free( (*cur) );
		}
	}
	(*cur) = NULL;

	CSDEBUG1( CS_INFO, "Done creating conn pool.  %d connection lists made.\n", i );
    if ( numConnectionsMade == 0 ) {
        WSASetLastError( WSASYSCALLFAILURE );
    }
	return( numConnectionsMade );
}

int
ReleaseConnPool( struct connpool* g_cp)
{

	struct connpool*	cur;
	struct connpool*	toFree;

	for ( cur = g_cp; cur != NULL; ) {
		toFree = cur;
		teardown_conn( cur->cp_cl );
		cur = cur->cp_next;
		free( toFree );
	}
	return( 0 );
}

/*
 * GrabConnection()
 *
 * Looks for a connection in the connection pool which is not currently
 * being used.  Returns a pointer to that connection, if availalbe.  Returns
 * NULL if no connection is available.
 */
struct connpool *
GrabConnection( struct connpool* cp )
{

	struct connpool*	cur;
	int	i = 0;

	for ( cur = cp; cur != NULL; cur = cur->cp_next ) {
		if ( !cur->cp_locked ) {
			cur->cp_locked = 1;
			CSDEBUG1( CS_INFO, "Grabbing connection %d.\n", i );
			return( cur );
		}
	}
    CosignLog( CS_ERROR, "GrabConnection: No currently available connections.\n" );
	return( NULL );
}

/*
 * SetCosignCerts()
 *
 */
int
SetCosignCerts( COSIGNSETTINGS*	cs, SSL_CTX	**g_ctx )
{
	SSL_CTX	*ctx = NULL;

	//SSL_load_error_strings();
    SSL_library_init();

	CSDEBUG0( CS_INFO, "SSL_ctx_new...\n" );
	if ( (ctx = SSL_CTX_new( SSLv23_client_method()) ) == NULL ) {
        CosignLog( CS_ERROR, "SetCosignCerts: SSL_CTX_new() failed.\n" );
		return( -1 );
	}

	CSDEBUG0( CS_INFO, "ssl_ctx_use_privatekey_file...\n" );
	if ( SSL_CTX_use_PrivateKey_file( ctx, cs->cs_privateKeyFilePath, SSL_FILETYPE_PEM ) != 1 ) {
        CosignLog( CS_ERROR, "SetCosignCerts: SSL_CTX_use_PrivateKey_file( %s ) failed.\n", cs->cs_privateKeyFilePath );
        return( -1 );
    }

	CSDEBUG0( CS_INFO, "ssl_ctx_use_certificate_chain_file...\n" );
	if ( SSL_CTX_use_certificate_chain_file( ctx, cs->cs_chainFilePath ) != 1 ) {
		CosignLog( CS_INFO, "SSL_CTX_use_certificate_chain_file( %s ) failed.\n",
			cs->cs_chainFilePath );
		return( -1 );
	}
	CSDEBUG0( CS_INFO, "ssl_ctx_check_private_key...\n" );
	if ( SSL_CTX_check_private_key( ctx ) != 1 ) {
		CosignLog( CS_ERROR, "SSL_CTX_check_private_key() failed.\n" );
		return( -1 );
	}

	CSDEBUG0( CS_INFO, "ssl_ctx_load_verify_locations...\n" );
	if ( SSL_CTX_load_verify_locations( ctx, NULL, cs->cs_CAFilePath ) != 1 ) {
		CosignLog( CS_ERROR, "SSL_CTX_load_verify_locations( %s ) failed.\n",
			cs->cs_CAFilePath );
		return( -1 );
	}
	CSDEBUG0( CS_INFO, "ssl_ctx_set_verify...\n" );
	SSL_CTX_set_verify( ctx, SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT, NULL );

	if ( ctx == NULL ) {
		CSDEBUG0( CS_INFO, "SetCosignCerts, ctx is NULL!\n" );
	} else {
		CSDEBUG0( CS_INFO, "SetCosignCerts, ctx is something!\n" );
	}
	*g_ctx = ctx;

	CSDEBUG0( CS_INFO, "setcosigncerts().  All done!\n" );
	return( 0 );
}

